@@ -58,7 +58,7 @@ package org.apache.oro.text.awk;
  */
 
 /*
- * $Id: AwkCompiler.java,v 1.5 2001/05/20 23:55:21 dfs Exp $
+ * $Id: AwkCompiler.java,v 1.7 2002/11/20 22:51:29 dfs Exp $
  */
 import org.apache.oro.text.regex.*;
 
@@ -172,6 +172,9 @@ public final class AwkCompiler implements PatternCompiler {
 
   static final char _END_OF_INPUT = '\uFFFF';
   
+  // All of these are initialized by the compile() and _parse() methods
+  // so there is no need or use in initializing them in the constructor
+  // although this may change in the future.
   private boolean __inCharacterClass, __caseSensitive, __multiline;
   private boolean __beginAnchor, __endAnchor;
   private char __lookahead;
@@ -179,7 +182,9 @@ public final class AwkCompiler implements PatternCompiler {
   private char[] __regularExpression;
   private int __openParen, __closeParen;
 
-  public AwkCompiler() { }
+  // We do not currently need to initialize any state, but keep this
+  // commented out as a reminder that we may have to at some point.
+  //public AwkCompiler() { }
 
   private static boolean __isMetachar(char token) {
     return (token == '*' || token == '?' || token == '+' ||
@@ -341,7 +346,6 @@ public final class AwkCompiler implements PatternCompiler {
   private SyntaxNode __repetition(SyntaxNode atom)
     throws MalformedPatternException {
     int min, max, startPosition[];
-    StringBuffer minBuffer, maxBuffer;
     SyntaxNode root = null;
     CatNode catNode;
 
