@@ -58,7 +58,7 @@ package org.apache.oro.text.awk;
  */
 
 /*
- * $Id: AwkMatcher.java,v 1.6 2001/07/10 12:39:18 dfs Exp $
+ * $Id: AwkMatcher.java,v 1.8 2002/11/20 21:31:03 dfs Exp $
  */
 import java.io.*;
 
@@ -621,12 +621,19 @@ public final class AwkMatcher implements PatternMatcher {
     __beginOffset = 0;
     _search();
     input._currentOffset = __lastMatchedBufferOffset;
-    return (__lastMatchResult != null);
+
+    if(__lastMatchResult != null) {
+      // Adjust match begin offset to be relative to beginning of stream.
+      __lastMatchResult._incrementMatchBeginOffset(input._bufferOffset);
+      return true;
+    }
+
+    return false;
   }
 
 
   private int __streamMatchPrefix() throws IOException {
-    int token, current = AwkPattern._START_STATE, lastState, transition;
+    int token, current = AwkPattern._START_STATE, lastState;
     int offset, initialOffset, maxOffset;
     int lastMatchedOffset = -1;
     int[] tstateArray;
@@ -689,9 +696,7 @@ public final class AwkMatcher implements PatternMatcher {
 
 
    void _search() throws IOException {
-    char[] currentLine;
     int position, tokensMatched;
-    String currentString;
 
     __lastMatchResult = null;
 
