@@ -58,11 +58,12 @@ package org.apache.oro.text.perl;
  */
 
 /*
- * $Id: Perl5Util.java,v 1.10 2001/12/08 03:15:25 dfs Exp $
+ * $Id: Perl5Util.java,v 1.15 2003/01/03 17:18:07 dfs Exp $
  */
 import java.util.*;
-import org.apache.oro.text.regex.*;
+
 import org.apache.oro.text.*;
+import org.apache.oro.text.regex.*;
 import org.apache.oro.util.*;
 
 /**
@@ -77,7 +78,7 @@ import org.apache.oro.util.*;
  * the slashes.
  *  <p>
  * The objective of the class is to minimize the amount of code a Java
- * programmer using OROMatcher<font size="-2"><sup>TM</sup></font>
+ * programmer using Jakarta-ORO
  * has to write to achieve the same results as Perl by 
  * transparently handling regular expression compilation, caching, and
  * matching.  A second objective is to use the same Perl pattern matching
@@ -86,7 +87,7 @@ import org.apache.oro.util.*;
  * All the state affecting methods are synchronized to avoid
  * the maintenance of explicit locks in multithreaded programs.  This
  * philosophy differs from the
- * OROMatcher<font size="-2"><sup>TM</sup></font> package, where
+ * {@link org.apache.oro.text.regex} package, where
  * you are expected to either maintain explicit locks, or more preferably
  * create separate compiler and matcher instances for each thread.
  * <p>
@@ -123,10 +124,12 @@ import org.apache.oro.util.*;
  * <p>
  * A couple of things to remember when using this class are that the
  * {@link #match match()} methods have the same meaning as
- * contains() in OROMatcher<font size="-2"><sup>TM</sup></font>
+ * {@link org.apache.oro.text.regex.Perl5Matcher#contains
+ *  Perl5Matcher.contains()}
  * and <code>=~ m/pattern/</code> in Perl.  The methods are named match
  * to more closely associate them with Perl and to differentiate them
- * from matches() in OROMatcher<font size="-2"><sup>TM</sup></font>.
+ * from {@link org.apache.oro.text.regex.Perl5Matcher#matches
+ * Perl5Matcher.matches()}.
  * A further thing to keep in mind is that the
  * {@link MalformedPerl5PatternException} class is derived from
  * RuntimeException which means you DON'T have to catch it.  The reasoning
@@ -143,12 +146,21 @@ import org.apache.oro.util.*;
  * programs to be robust.
  * <p>
  * Finally, as a convenience Perl5Util implements 
- * the org.apache.oro.text.regex.MatchResult interface found in the
- * OROMatcher<font size="-2"><sup>TM</sup></font> package.  The methods
- * are merely wrappers which call the corresponding method of the last
- * MatchResult found (which can be accessed with
- * {@link #getMatch()} by a match or substitution
- * (or even a split, but this isn't particularly useful).
+ * the {@link org.apache.oro.text.regex.MatchResult MatchResult} interface.
+ * The methods are merely wrappers which call the corresponding method of
+ * the last {@link org.apache.oro.text.regex.MatchResult MatchResult}
+ * found (which can be accessed with {@link #getMatch()} by a match or
+ * substitution (or even a split, but this isn't particularly useful).
+ * At the moment, the
+ * {@link org.apache.oro.text.regex.MatchResult MatchResult} returned
+ * by {@link #getMatch()} is not stored in a thread-local variable.  Therefore
+ * concurrent calls to {@link #getMatch()} will produce unpredictable
+ * results.  So if your concurrent program requires the match results,
+ * you must protect the matching and the result retrieval in a critical
+ * section.  If you do not need match results, you don't need to do anything
+ * special.  If you feel the J2SE implementation of {@link #getMatch()}
+ * should use a thread-local variable and obviate the need for a critical
+ * section, please express your views on the oro-dev mailing list.
  *
  * @author <a href="mailto:oro-dev@jakarta.apache.org">Daniel F. Savarese</a>
  * @version @version@
@@ -167,11 +179,16 @@ public final class Perl5Util implements MatchResult {
   /** The hashtable to cache higher-level expressions */
   private Cache __expressionCache;
   /** The pattern matcher to perform matching operations. */
-  private Perl5Matcher __matcher = new Perl5Matcher();
+  private Perl5Matcher __matcher;
   /** The compiled match expression parsing regular expression. */
   private Pattern __matchPattern;
   /** The last match from a successful call to a matching method. */
   private MatchResult __lastMatch;
+  /**
+   * A container for temporarily holding the results of a split before
+   * deleting trailing empty fields.
+   */
+  private ArrayList __splitList;
 
   /**
    * Keeps track of the original input (for postMatch() and preMatch())
@@ -215,6 +232,8 @@ public final class Perl5Util implements MatchResult {
    * </pre>
    */
   public Perl5Util(PatternCache cache) {
+    __splitList    = new ArrayList();
+    __matcher      = new Perl5Matcher();
     __patternCache = cache;
     __expressionCache = new CacheLRU(cache.capacity());
     __compilePatterns();
@@ -384,7 +403,8 @@ public final class Perl5Util implements MatchResult {
    * As with Perl, any non-alphanumeric character can be used in lieu of
    * the slashes.
    * <p>
-   * If the input contains the pattern, the org.apache.oro.text.regex.MatchResult
+   * If the input contains the pattern, the
+   * {@link org.apache.oro.text.regex.MatchResult MatchResult}
    * can be obtained by calling {@link #getMatch()}.
    * However, Perl5Util implements the MatchResult interface as a wrapper
    * around the last MatchResult found, so you can call its methods to
@@ -423,7 +443,8 @@ public final class Perl5Util implements MatchResult {
    * As with Perl, any non-alphanumeric character can be used in lieu of
    * the slashes.
    * <p>
-   * If the input contains the pattern, the org.apache.oro.text.regex.MatchResult
+   * If the input contains the pattern, the
+   * {@link org.apache.oro.text.regex.MatchResult MatchResult}
    * can be obtained by calling {@link #getMatch()}.
    * However, Perl5Util implements the MatchResult interface as a wrapper
    * around the last MatchResult found, so you can call its methods to
@@ -431,7 +452,8 @@ public final class Perl5Util implements MatchResult {
    * After the call to this method, the PatternMatcherInput current offset
    * is advanced to the end of the match, so you can use it to repeatedly
    * search for expressions in the entire input using a while loop as
-   * explained in the OROMatcher<font size="-2"><sup>TM</sup></font> package.
+   * explained in the {@link org.apache.oro.text.regex.PatternMatcherInput
+   * PatternMatcherInput} documentation.
    * <p>
    * @param pattern  The pattern to search for.
    * @param input    The PatternMatcherInput to search.
@@ -489,13 +511,14 @@ public final class Perl5Util implements MatchResult {
    * <dt> m <dd> treat the input as consisting of multiple lines
    * <dt> o <dd> If variable interopolation is used, only evaluate the
    *             interpolation once (the first time).  This is equivalent
-   *             to using a numInterpolations argument of 1 in the 
-   *             OROMatcher<font size="-2"><sup>TM</sup></font> 
-   *             Util.substitute() method.  The default is to compute
-   *             each interpolation independently.  See the
-   *             OROMatcher<font size="-2"><sup>TM</sup></font>
-   *             Util.substitute() method for more details on variable
-   *             interpolation in substitutions.
+   *             to using a numInterpolations argument of 1 in
+   * {@link org.apache.oro.text.regex.Util#substitute Util.substitute()}.
+   *             The default is to compute each interpolation independently.
+   *             See
+   * {@link org.apache.oro.text.regex.Util#substitute Util.substitute()}
+   * and {@link org.apache.oro.text.regex.Perl5Substitution Perl5Substitution}
+   *             for more details on variable interpolation in
+   *             substitutions.
    * <dt> s <dd> treat the input as consisting of a single line
    * <dt> x <dd> enable extended expression syntax incorporating whitespace
    *             and comments
@@ -504,11 +527,11 @@ public final class Perl5Util implements MatchResult {
    * the slashes.  This is helpful to avoid backslashing.  For example,
    * using slashes you would have to do:
    * <blockquote><pre>
-   * result = util.substitute("s/foo\\/bar/goo\\/\\/baz/", input);
+   * numSubs = util.substitute(result, "s/foo\\/bar/goo\\/\\/baz/", input);
    * </pre></blockquote>
    * when you could more easily write:
    * <blockquote><pre>
-   * result = util.substitute("s#foo/bar#goo//baz#", input);
+   * numSubs = util.substitute(result, "s#foo/bar#goo//baz#", input);
    * </pre></blockquote>
    * where the hashmarks are used instead of slashes.
    * <p>
@@ -535,25 +558,28 @@ public final class Perl5Util implements MatchResult {
    * used to be an invalid expression and require a special-case extra
    * backslash, will now replace all instances of / with \:
    * <blockquote><pre>
-   * result = util.substitute("s#/#\\#g", input);
+   * numSubs = util.substitute(result, "s#/#\\#g", input);
    * </pre></blockquote>
    * <p>
-   * @param expression The substitution expression.
-   * @param input      The input.
-   * @return           The input after substitutions have been performed.
+   * @param result     The StringBuffer in which to store the result of the
+   *                   substitutions. The buffer is only appended to.
+   * @param expression The Perl5 substitution regular expression.
+   * @param input      The input on which to perform substitutions.
+   * @return The number of substitutions made.
    * @exception MalformedPerl5PatternException  If there is an error in
    *            the expression.  You are not forced to catch this exception
    *            because it is derived from RuntimeException.
+   * @since 2.0.6
    */
   // Expression parsing will have to be moved into a separate method if
   // there are going to be variations of this method.
-  public synchronized String substitute(String expression, String input)
+  public synchronized int substitute(StringBuffer result, String expression,
+				     String input)
        throws MalformedPerl5PatternException 
   {
     boolean backslash, finalDelimiter;
     int index, compileOptions, numSubstitutions, numInterpolations;
-    int firstOffset, secondOffset, thirdOffset;
-    String result;
+    int firstOffset, secondOffset, thirdOffset, subCount;
     StringBuffer replacement;
     Pattern compiledPattern;
     char exp[], delimiter;
@@ -575,12 +601,14 @@ public final class Perl5Util implements MatchResult {
 	break __nullTest;
       }
 
-      result = Util.substitute(__matcher, entry._pattern, entry._substitution,
-			       input, entry._numSubstitutions);
+
+      subCount =
+	Util.substitute(result, __matcher, entry._pattern, entry._substitution,
+			input, entry._numSubstitutions);
 
       __lastMatch = __matcher.getMatch();
 
-      return result;
+      return subCount;
     }
 
     exp = expression.toCharArray();
@@ -680,15 +708,46 @@ public final class Perl5Util implements MatchResult {
 					numSubstitutions);
     __expressionCache.addElement(expression, entry);
 
-    result = Util.substitute(__matcher, compiledPattern, substitution,
-			     input, numSubstitutions);
+    subCount =
+      Util.substitute(result, __matcher, compiledPattern, substitution,
+		      input, numSubstitutions);
 
     __lastMatch = __matcher.getMatch();
 
-    return result;
+    return subCount;
   }
 
-
+  /**
+   * Substitutes a pattern in a given input with a replacement string.
+   * The substitution expression is specified in Perl5 native format.
+   * <dl compact>
+   *   <dt>Calling this method is the same as:</dt>
+   *   <dd>
+   *     <blockquote><pre>
+   *      String result;
+   *      StringBuffer buffer = new StringBuffer();
+   *      perl.substitute(buffer, expression, input);
+   *      result = buffer.toString();
+   *     </pre></blockquote>
+   *   </dd>
+   * </dl>
+   * @param expression The Perl5 substitution regular expression.
+   * @param input      The input on which to perform substitutions.
+   * @return  The input as a String after substitutions have been performed.
+   * @exception MalformedPerl5PatternException  If there is an error in
+   *            the expression.  You are not forced to catch this exception
+   *            because it is derived from RuntimeException.
+   * @since 1.0
+   * @see #substitute
+   */
+  public synchronized String substitute(String expression, String input)
+    throws MalformedPerl5PatternException
+  {
+    StringBuffer result = new StringBuffer();
+    substitute(result, expression, input);
+    return result.toString();
+  }
+ 
   /**
    * Splits a String into strings that are appended to a List, but no more
    * than a specified limit.  The String is split using a regular expression
@@ -724,20 +783,25 @@ public final class Perl5Util implements MatchResult {
    * <blockquote><pre>
    * { "8", "-", "12", ",", "15", ",", "18" }
    * </pre></blockquote>
-   * The Util.split() method in the
-   * OROMatcher<font size="-2"><sup>TM</sup></font> package does NOT
-   * implement this particular behavior because it is intended to
-   * be usable with Pattern instances other than Perl5Pattern.
+   * Furthermore, the following Perl behavior is observed: "leading empty
+   * fields are preserved, and empty trailing one are deleted."  This
+   * has the effect that a split on a zero length string returns an empty
+   * list.
+   * The {@link org.apache.oro.text.regex.Util#split Util.split()} method
+   * does NOT implement these behaviors because it is intended to
+   * be a general self-consistent and predictable split function usable
+   * with Pattern instances other than Perl5Pattern.
    * <p>
    * @param results 
-   *    A <code> List </code> to which the substrings of the input
+   *    A <code> Collection </code> to which the substrings of the input
    *    that occur between the regular expression delimiter occurences
    *    are appended. The input will not be split into any more substrings
    *    than the specified 
    *    limit. A way of thinking of this is that only the first
    *    <b>limit - 1</b>
    *    matches of the delimiting regular expression will be used to split the
-   *    input. 
+   *    input.  The Collection must support the
+   *    <code>addAll(Collection)</code> operation.
    * @param pattern The regular expression to use as a split delimiter.
    * @param input The String to split.
    * @param limit The limit on the size of the returned <code>Vector</code>.
@@ -768,20 +832,35 @@ public final class Perl5Util implements MatchResult {
     while(--limit != 0 && __matcher.contains(pinput, compiledPattern)) {
       currentResult = __matcher.getMatch();
 
-      results.add(input.substring(beginOffset,
-				  currentResult.beginOffset(0)));
+      __splitList.add(input.substring(beginOffset,
+				      currentResult.beginOffset(0)));
+
       if((groups = currentResult.groups()) > 1) {
 	for(index = 1; index < groups; ++index) {
 	  group = currentResult.group(index);
 	  if(group != null && group.length() > 0)
-	    results.add(group);
+	    __splitList.add(group);
 	}
       }
 
       beginOffset = currentResult.endOffset(0);
     }
 
-    results.add(input.substring(beginOffset, input.length()));
+    __splitList.add(input.substring(beginOffset, input.length()));
+
+    // Remove all trailing empty fields.
+    for(int i = __splitList.size() - 1; i >= 0; --i) {
+      String str;
+
+      str = (String)__splitList.get(i);
+      if(str.length() == 0)
+	__splitList.remove(i);
+      else
+	break;
+    }
+
+    results.addAll(__splitList);
+    __splitList.clear();
 
     // Just for the sake of completeness
     __lastMatch = currentResult;
@@ -848,9 +927,8 @@ public final class Perl5Util implements MatchResult {
    * <blockquote><pre>
    * { "8", "-", "12", ",", "15", ",", "18" }
    * </pre></blockquote>
-   * The Util.split() method in the
-   * OROMatcher<font size="-2"><sup>TM</sup></font> package does NOT
-   * implement this particular behavior because it is intended to
+   * The {@link org.apache.oro.text.regex.Util#split Util.split()} method
+   * does NOT implement this particular behavior because it is intended to
    * be usable with Pattern instances other than Perl5Pattern.
    * <p>
    * @deprecated Use
@@ -1018,12 +1096,14 @@ public final class Perl5Util implements MatchResult {
    * @return A string containing the entire match.
    */  
   public synchronized String toString() {
+    if(__lastMatch == null)
+      return null;
     return __lastMatch.toString();
   }
 
 
   /**
-   * Returns the part of the input preceding that last match found.
+   * Returns the part of the input preceding the last match found.
    * <p>
    * @return The part of the input following the last match found.
    */
@@ -1067,7 +1147,7 @@ public final class Perl5Util implements MatchResult {
 
 
   /**
-   * Returns the part of the input following that last match found.
+   * Returns the part of the input following the last match found.
    * <p>
    * @return The part of the input following the last match found.
    */
@@ -1110,7 +1190,7 @@ public final class Perl5Util implements MatchResult {
 
 
   /**
-   * Returns the part of the input preceding that last match found as a
+   * Returns the part of the input preceding the last match found as a
    * char array.  This method eliminates the extra
    * buffer copying caused by preMatch().toCharArray().
    * <p>
@@ -1161,7 +1241,7 @@ public final class Perl5Util implements MatchResult {
 
 
   /**
-   * Returns the part of the input following that last match found as a char
+   * Returns the part of the input following the last match found as a char
    * array.  This method eliminates the extra buffer copying caused by
    * preMatch().toCharArray().
    * <p>
