@@ -58,7 +58,7 @@ package org.apache.oro.text.regex;
  */
 
 /*
- * $Id: Perl5Compiler.java,v 1.13 2001/10/23 06:42:20 dfs Exp $
+ * $Id: Perl5Compiler.java,v 1.18 2003/01/03 17:18:07 dfs Exp $
  */
 import java.util.*;
 
@@ -68,6 +68,16 @@ import java.util.*;
  * Perl5Pattern instances upon compilation to be used in conjunction
  * with a Perl5Matcher instance.  Please see the user's guide for more 
  * information about Perl5 regular expressions.
+ * <p>
+ * Perl5Compiler and Perl5Matcher are designed with the intent that
+ * you use a separate instance of each per thread to avoid the overhead
+ * of both synchronization and concurrent access (e.g., a match that takes
+ * a long time in one thread will block the progress of another thread with
+ * a shorter match).  If you want to use a single instance of each
+ * in a concurrent program, you must appropriately protect access to
+ * the instances with critical sections.  If you want to share Perl5Pattern
+ * instances between concurrently executing instances of Perl5Matcher, you
+ * must compile the patterns with {@link Perl5Compiler#READ_ONLY_MASK}.
  *
  * @author <a href="mailto:oro-dev@jakarta.apache.org">Daniel F. Savarese</a>
  * @version @version@
@@ -91,7 +101,6 @@ public final class Perl5Compiler implements PatternCompiler {
     __EXTENDED         = 0x0020,
     __READ_ONLY        = 0x8000;
 
-  private static final String __META_CHARS = "^$.[()|?+*\\";
   private static final String __HEX_DIGIT =
   "0123456789abcdef0123456789ABCDEFx";
   private CharStringPointer __input;
@@ -312,7 +321,7 @@ public final class Perl5Compiler implements PatternCompiler {
   private static int __parseOctal(char[] str, int offset, int maxLength,
 				 int[] scanned)
   {
-    int val = 0, index;
+    int val = 0;
 
     scanned[0] = 0;
     while(offset < str.length && 
@@ -938,7 +947,10 @@ public final class Perl5Compiler implements PatternCompiler {
     return offset;
   }
 
-
+  // These are the original 8-bit character class handling methods.
+  // We don't want to delete them just yet only to have to dig it out
+  // of revision control later.
+  /*
   // Set the bits in a character class.  Only recognizes ascii.
   private void __setCharacterClassBits(char[] bits, int offset, char deflt,
 				       char ch)
@@ -954,7 +966,6 @@ public final class Perl5Compiler implements PatternCompiler {
     }
   }
 
-
   private int __parseCharacterClass() throws MalformedPatternException {
     boolean range = false, skipTest;
     char clss, deflt, lastclss = Character.MAX_VALUE;
@@ -1104,7 +1115,7 @@ public final class Perl5Compiler implements PatternCompiler {
 
     return offset;
   }
-
+  */
 
   private int __parseUnicodeClass() throws MalformedPatternException {
     boolean range = false, skipTest;
@@ -1524,11 +1535,12 @@ public final class Perl5Compiler implements PatternCompiler {
   private int __parseExpression(boolean isParenthesized, int[] hintFlags)
     throws MalformedPatternException {
     char value, paren;
+    char[] modifierFlags, posFlags = { 0 }, negFlags = { 0 };
     int nodeOffset = OpCode._NULL_OFFSET, parenthesisNum = 0, br, ender;
-    int[] flags = { 0 };
-    String modifiers = "iogmsx";
-
+    int[] flags = { 0 };;
+    String modifiers = "iogmsx-";
 
+    modifierFlags = posFlags;
     // Initially we assume expression doesn't match null string.
     hintFlags[0] = __NONNULL;
 
@@ -1557,9 +1569,15 @@ public final class Perl5Compiler implements PatternCompiler {
 	  value = __input._getValue();
 	  while(value != CharStringPointer._END_OF_STRING &&
 		modifiers.indexOf(value) != -1) {
-	    __setModifierFlag(__modifierFlags, value);
+	    if(value == '-')
+	      modifierFlags = negFlags;
+	    else
+	      __setModifierFlag(modifierFlags, value);
 	    value = __input._increment();
 	  }
+	  __modifierFlags[0] |= posFlags[0];
+	  __modifierFlags[0] &= ~negFlags[0];
+
 	  if(value != ')')
 	    throw new MalformedPatternException(
 	       "Sequence (?" + value + "...) not recognized");
