@@ -59,15 +59,24 @@ package org.apache.oro.text.regex;
 
 
 /*
- * $Id: Perl5Matcher.java,v 1.17 2001/11/14 02:39:37 dfs Exp $
+ * $Id: Perl5Matcher.java,v 1.22 2003/01/03 17:18:08 dfs Exp $
  */
-import java.io.IOException;
 import java.util.*;
 
 /**
  * The Perl5Matcher class is used to match regular expressions
  * (conforming to the Perl5 regular expression syntax) generated by
  * Perl5Compiler.
+ * <p>
+ * Perl5Compiler and Perl5Matcher are designed with the intent that
+ * you use a separate instance of each per thread to avoid the overhead
+ * of both synchronization and concurrent access (e.g., a match that takes
+ * a long time in one thread will block the progress of another thread with
+ * a shorter match).  If you want to use a single instance of each
+ * in a concurrent program, you must appropriately protect access to
+ * the instances with critical sections.  If you want to share Perl5Pattern
+ * instances between concurrently executing instances of Perl5Matcher, you
+ * must compile the patterns with {@link Perl5Compiler#READ_ONLY_MASK}.
  *
  * @author <a href="mailto:oro-dev@jakarta.apache.org">Daniel F. Savarese</a>
  * @version @version@
@@ -241,7 +250,7 @@ public final class Perl5Matcher implements PatternMatcher {
   // Set the match result information.  Only call this if we successfully
   // matched.
   private void __setLastMatchResult() {
-    int offs;
+    int offs, maxEndOffs = 0;
 
     //endOffset+=dontTry;
 
@@ -251,34 +260,36 @@ public final class Perl5Matcher implements PatternMatcher {
     if(__endMatchOffsets[0] > __originalInput.length)
       throw new ArrayIndexOutOfBoundsException();
 
-    __lastMatchResult._match =
-      new String(__originalInput, __beginMatchOffsets[0],
-		 __endMatchOffsets[0] - __beginMatchOffsets[0]);
-
     __lastMatchResult._matchBeginOffset = __beginMatchOffsets[0];
 
     while(__numParentheses >= 0) {
       offs = __beginMatchOffsets[__numParentheses];
 
       if(offs >= 0)
-	__lastMatchResult._beginGroupOffset[__numParentheses]
-	  = offs - __lastMatchResult._matchBeginOffset;
+	__lastMatchResult._beginGroupOffset[__numParentheses] =
+	  offs - __lastMatchResult._matchBeginOffset;
       else
 	__lastMatchResult._beginGroupOffset[__numParentheses] =
 	  OpCode._NULL_OFFSET;
 
       offs = __endMatchOffsets[__numParentheses];
 
-      if(offs >= 0)
-	__lastMatchResult._endGroupOffset[__numParentheses]
-	  = offs - __lastMatchResult._matchBeginOffset;
-      else
+      if(offs >= 0) {
+	__lastMatchResult._endGroupOffset[__numParentheses] =
+	  offs - __lastMatchResult._matchBeginOffset;
+	if(offs > maxEndOffs && offs <= __originalInput.length)
+	  maxEndOffs = offs;
+      } else
 	__lastMatchResult._endGroupOffset[__numParentheses] =
 	  OpCode._NULL_OFFSET;
 
       --__numParentheses;
     }
 
+    __lastMatchResult._match =
+      new String(__originalInput, __beginMatchOffsets[0],
+		 maxEndOffs - __beginMatchOffsets[0]);
+
     // Free up for garbage collection
     __originalInput = null;
   }
@@ -343,32 +354,30 @@ public final class Perl5Matcher implements PatternMatcher {
       }
 
       if((expression._anchor & Perl5Pattern._OPT_ANCH) != 0) {
-	if(__currentOffset == beginOffset &&
-	   __tryExpression(expression, beginOffset))
+	if(__currentOffset == beginOffset && __tryExpression(beginOffset)) {
+	  success = true;
+	  break _mainLoop;
+	} else if(__multiline
+		  || (expression._anchor & Perl5Pattern._OPT_ANCH_MBOL) != 0
+		  || (expression._anchor & Perl5Pattern._OPT_IMPLICIT) != 0)
 	  {
-	    success = true;
-	    break _mainLoop;
-	  } else if(__multiline
-		    || (expression._anchor & Perl5Pattern._OPT_ANCH_MBOL) != 0
-		    || (expression._anchor & Perl5Pattern._OPT_IMPLICIT) != 0)
-	    {
-	      if(minLength > 0)
-		dontTry = minLength - 1;
-	      endOffset-=dontTry;
+	    if(minLength > 0)
+	      dontTry = minLength - 1;
+	    endOffset-=dontTry;
 
-	      if(__currentOffset > currentOffset)
-		--__currentOffset;
+	    if(__currentOffset > currentOffset)
+	      --__currentOffset;
 
-	      while(__currentOffset < endOffset) {
-		if(__input[__currentOffset++] == '\n') {
-		  if(__currentOffset < endOffset &&
-		     __tryExpression(expression, __currentOffset)) {
-		    success = true;
-		    break _mainLoop;
-		  }
+	    while(__currentOffset < endOffset) {
+	      if(__input[__currentOffset++] == '\n') {
+		if(__currentOffset < endOffset &&
+		   __tryExpression(__currentOffset)) {
+		  success = true;
+		  break _mainLoop;
 		}
 	      }
 	    }
+	  }
 
 	break _mainLoop;
       }
@@ -381,7 +390,7 @@ public final class Perl5Matcher implements PatternMatcher {
 
 	  while(__currentOffset < endOffset) {
 	    if(ch == __input[__currentOffset]) {
-	      if(__tryExpression(expression, __currentOffset)){
+	      if(__tryExpression(__currentOffset)){
 		success = true;
 		break _mainLoop;
 	      }
@@ -397,7 +406,7 @@ public final class Perl5Matcher implements PatternMatcher {
 	  while((__currentOffset =
 		 __findFirst(__input, __currentOffset, endOffset, mustString))
 		< endOffset){
-	    if(__tryExpression(expression, __currentOffset)) {
+	    if(__tryExpression(__currentOffset)) {
 	      success = true;
 	      break _mainLoop;
 	    }
@@ -427,7 +436,7 @@ public final class Perl5Matcher implements PatternMatcher {
 
 	    if(ch < 256 &&
 	       (__program[offset + (ch >> 4)] & (1 << (ch & 0xf))) == 0) {
-	      if(tmp && __tryExpression(expression, __currentOffset)) {
+	      if(tmp && __tryExpression(__currentOffset)) {
 		success = true;
 		break _mainLoop;
 	      } else
@@ -446,7 +455,7 @@ public final class Perl5Matcher implements PatternMatcher {
 	    ch = __input[__currentOffset];
 
 	    if(__matchUnicodeClass(ch, __program, offset, op)) {
-	      if(tmp && __tryExpression(expression, __currentOffset)) {
+	      if(tmp && __tryExpression(__currentOffset)) {
 		success = true;
 		break _mainLoop;
 	      } else
@@ -474,7 +483,7 @@ public final class Perl5Matcher implements PatternMatcher {
 	    ch = __input[__currentOffset];
 	    if(tmp != OpCode._isWordCharacter(ch)){
 	      tmp = !tmp;
-	      if(__tryExpression(expression, __currentOffset)) {
+	      if(__tryExpression(__currentOffset)) {
 		success = true;
 		break _mainLoop;
 	      }
@@ -483,7 +492,7 @@ public final class Perl5Matcher implements PatternMatcher {
 	  }
 
 	  if((minLength > 0 || tmp) && 
-	     __tryExpression(expression, __currentOffset)) {
+	     __tryExpression(__currentOffset)) {
 	    success = true;
 	    break _mainLoop;
 	  }
@@ -505,7 +514,7 @@ public final class Perl5Matcher implements PatternMatcher {
 	    ch = __input[__currentOffset];
 	    if(tmp != OpCode._isWordCharacter(ch))
 	      tmp = !tmp;
-	    else if(__tryExpression(expression, __currentOffset)) {
+	    else if(__tryExpression(__currentOffset)) {
 	      success = true;
 	      break _mainLoop;
 	    }
@@ -514,7 +523,7 @@ public final class Perl5Matcher implements PatternMatcher {
 	  }
 
 	  if((minLength > 0 || !tmp) &&
-	     __tryExpression(expression, __currentOffset)) {
+	     __tryExpression(__currentOffset)) {
 	    success = true;
 	    break _mainLoop;
 	  }
@@ -524,7 +533,7 @@ public final class Perl5Matcher implements PatternMatcher {
 	  while(__currentOffset < endOffset) {
 	    ch = __input[__currentOffset];
 	    if(OpCode._isWordCharacter(ch)) {
-	      if(tmp && __tryExpression(expression, __currentOffset)) {
+	      if(tmp && __tryExpression(__currentOffset)) {
 		success = true;
 		break _mainLoop;
 	      } else
@@ -539,7 +548,7 @@ public final class Perl5Matcher implements PatternMatcher {
 	  while(__currentOffset < endOffset) {
 	    ch = __input[__currentOffset];
 	    if(!OpCode._isWordCharacter(ch)) {
-	      if(tmp && __tryExpression(expression, __currentOffset)) {
+	      if(tmp && __tryExpression(__currentOffset)) {
 		success = true;
 		break _mainLoop;
 	      } else
@@ -553,7 +562,7 @@ public final class Perl5Matcher implements PatternMatcher {
 	case OpCode._SPACE:
 	  while(__currentOffset < endOffset) {
 	    if(Character.isWhitespace(__input[__currentOffset])) {
-	      if(tmp && __tryExpression(expression, __currentOffset)) {
+	      if(tmp && __tryExpression(__currentOffset)) {
 		success = true;
 		break _mainLoop;
 	      } else
@@ -567,7 +576,7 @@ public final class Perl5Matcher implements PatternMatcher {
 	case OpCode._NSPACE:
 	  while(__currentOffset < endOffset) {
 	    if(!Character.isWhitespace(__input[__currentOffset])) {
-	      if(tmp && __tryExpression(expression, __currentOffset)) {
+	      if(tmp && __tryExpression(__currentOffset)) {
 		success = true;
 		break _mainLoop;
 	      } else
@@ -581,7 +590,7 @@ public final class Perl5Matcher implements PatternMatcher {
 	case OpCode._DIGIT:
 	  while(__currentOffset < endOffset) {
 	    if(Character.isDigit(__input[__currentOffset])) {
-	      if(tmp && __tryExpression(expression, __currentOffset)) {
+	      if(tmp && __tryExpression(__currentOffset)) {
 		success = true;
 		break _mainLoop;
 	      } else
@@ -596,7 +605,7 @@ public final class Perl5Matcher implements PatternMatcher {
 	case OpCode._NDIGIT:
 	  while(__currentOffset < endOffset) {
 	    if(!Character.isDigit(__input[__currentOffset])) {
-	      if(tmp && __tryExpression(expression, __currentOffset)) {
+	      if(tmp && __tryExpression(__currentOffset)) {
 		success = true;
 		break _mainLoop;
 	      } else
@@ -614,7 +623,7 @@ public final class Perl5Matcher implements PatternMatcher {
 	endOffset-=dontTry;
 
 	do {
-	  if(__tryExpression(expression, __currentOffset)) {
+	  if(__tryExpression(__currentOffset)) {
 	    success = true;
 	    break _mainLoop;
 	  }
@@ -737,7 +746,7 @@ public final class Perl5Matcher implements PatternMatcher {
     return !isANYOF;
   }
   
-  private boolean __tryExpression(Perl5Pattern expression, int offset) {
+  private boolean __tryExpression(int offset) {
     int count;
     
     __inputOffset = offset;
@@ -1421,7 +1430,7 @@ public final class Perl5Matcher implements PatternMatcher {
 
     __initInterpreterGlobals(expression, input, 0, input.length, offset);
 
-    __lastSuccess = __tryExpression(expression, offset);
+    __lastSuccess = __tryExpression(offset);
     __lastMatchResult = null;
 
     return __lastSuccess;
@@ -1499,7 +1508,7 @@ public final class Perl5Matcher implements PatternMatcher {
 
     __initInterpreterGlobals(expression, inp, input._beginOffset,
 			     input._endOffset, input._currentOffset);
-    __lastSuccess = __tryExpression(expression, input._currentOffset);
+    __lastSuccess = __tryExpression(input._currentOffset);
     __lastMatchResult = null;
 
     return __lastSuccess;
@@ -1554,7 +1563,7 @@ public final class Perl5Matcher implements PatternMatcher {
       input = _toLower(input);
 
     __initInterpreterGlobals(expression, input, 0, input.length, 0);
-    __lastSuccess = (__tryExpression(expression, 0) &&
+    __lastSuccess = (__tryExpression(0) &&
 		     __endMatchOffsets[0] == input.length);
     __lastMatchResult = null;
 
@@ -1667,7 +1676,7 @@ public final class Perl5Matcher implements PatternMatcher {
 
     __lastMatchResult = null;
 
-    if(__tryExpression(expression, input._beginOffset)) {
+    if(__tryExpression(input._beginOffset)) {
       if(__endMatchOffsets[0] == input._endOffset ||
 	 input.length() == 0 || input._beginOffset == input._endOffset) {
 	__lastSuccess = true;
