@@ -58,7 +58,7 @@ package org.apache.oro.text.regex;
  */
 
 /*
- * $Id: Perl5Substitution.java,v 1.6 2001/05/17 21:16:11 dfs Exp $
+ * $Id: Perl5Substitution.java,v 1.10 2002/11/20 21:31:03 dfs Exp $
  */
 import java.util.*;
 
@@ -71,7 +71,7 @@ import java.util.*;
  * The substitution string may contain variable interpolations referring
  * to the saved parenthesized groups of the search pattern.
  * A variable interpolation is denoted by <b>$1</b>, or <b>$2</b>,
- * or <b>$3</b>, etc.  If you do not want such expressions to be
+ * or <b>$3</b>, etc.  If you want such expressions to be
  * interpreted literally, you should set the <b> numInterpolations </b>
  * parameter to <b> INTERPOLATE_NONE </b>.  It is easiest to explain
  * what an interpolated variable does by giving an example:
@@ -112,9 +112,14 @@ import java.util.*;
  * example, and a substitution expression of <b>a$2-</b>, the result
  * of the last sample input would be:
  * <pre><b>Tank a- 85  Tank a- 32  Tank a- 22</b></pre>
- * Also, the result of substituting <b>$0</b> or $ followed by an
- * non-positive integer is undefined.  In order to include a $ in a
- * substitution, it should be escaped with a backslash (e.g., <b>"\\$0"</b>).
+ * The special substitution <b>$&</b> will interpolate the entire portion
+ * of the input matched by the regular expression.  <b>$0</b> will
+ * do the same, but it is recommended that it be avoided because the
+ * latest versions of Perl use <b>$0</b> to store the program name rather
+ * than duplicate the behavior of <b>$&</b>.
+ * Also, the result of substituting $ followed by a non-positive integer
+ * is undefined.  In order to include a $ in a  substitution, it should
+ * be escaped with a backslash (e.g., <b>"\\$0"</b>).
  * <p>
  * Perl5 double-quoted string case modification is also supported in
  * the substitution.  The following escape sequences are supported:
@@ -131,6 +136,7 @@ import java.util.*;
  *
  * @author <a href="mailto:oro-dev@jakarta.apache.org">Daniel F. Savarese</a>
  * @author <a href="mailto:oro-dev@jakarta.apache.org">Mark Murphy</a>
+ * @author <a href="mailto:oro-dev@jakarta.apache.org">Takashi Okamoto</a>
  * @version @version@
  * @since 1.1
  * @see Substitution
@@ -202,6 +208,10 @@ public class Perl5Substitution extends StringSubstitution {
 
   transient String _lastInterpolation;
 
+  private static final boolean __isInterpolationCharacter(char ch) {
+    return (Character.isDigit(ch) || ch == '&');
+  }
+
   private void __addElement(int value) {
     int len = _subOpcodes.length;
     if (_subOpcodesCount == len) {
@@ -212,7 +222,7 @@ public class Perl5Substitution extends StringSubstitution {
     _subOpcodes[_subOpcodesCount++] = value;
   }
 
-  private void _parseSubs(String sub) {
+  private void __parseSubs(String sub) {
     boolean saveDigits, escapeMode, caseMode;
     int posParam;
     int offset;
@@ -237,6 +247,7 @@ public class Perl5Substitution extends StringSubstitution {
       // Save digits
       if (saveDigits) {
 	int digit = Character.digit(c, 10);
+
 	if (digit > -1) {
 	  if (posParam <= __MAX_GROUPS) {
 	    posParam *= 10;
@@ -246,11 +257,19 @@ public class Perl5Substitution extends StringSubstitution {
 	    __addElement(posParam);
 	  }
 	  continue;
+	} else if(c == '&') {
+	  if(/*current > 0 &&*/subChars[current - 1] == '$') {
+	    __addElement(0);
+	    posParam = 0;
+	    saveDigits = false;
+	    continue;
+	  }
 	}
+
 	__addElement(posParam);
 	posParam = 0;
 	saveDigits = false;
-      }
+      } 
 
       if ((c != '$' && c != '\\') || escapeMode) {
 	escapeMode = false;
@@ -278,11 +297,8 @@ public class Perl5Substitution extends StringSubstitution {
 
       // Positional params
       if (c == '$') {
-	saveDigits = (nextc != '0' && Character.isDigit(nextc));
-      }
-
-      // Escape codes
-      else if (c == '\\') {
+	saveDigits = __isInterpolationCharacter(nextc);
+      } else if (c == '\\') { // Escape codes
 	if (nextc == 'l') {
 	  if (!caseMode){
 	    __addElement(_OPCODE_LOWERCASE_CHAR);
@@ -321,7 +337,6 @@ public class Perl5Substitution extends StringSubstitution {
 
   void _calcSub(StringBuffer buffer, MatchResult result) {
     int size, offset, count, caseMode;
-    String group;
     char[] sub, str, match;
     int[] subOpcodes = _subOpcodes;
 
@@ -337,7 +352,7 @@ public class Perl5Substitution extends StringSubstitution {
 
       // If we have a group, set up interpolation, else
       // interpret op code.
-      if(value > 0 && value < result.groups()) {
+      if(value >= 0 && value < result.groups()) {
 	int end, len;
 	offset = result.begin(value);
 	
@@ -486,7 +501,7 @@ public class Perl5Substitution extends StringSubstitution {
 
     if(numInterpolations != INTERPOLATE_NONE && 
        (substitution.indexOf('$') != -1 || substitution.indexOf('\\') != -1))
-      _parseSubs(substitution);
+      __parseSubs(substitution);
     else
       _subOpcodes = null;
     _lastInterpolation = null;
