@@ -58,7 +58,7 @@ package org.apache.oro.text.regex;
  */
 
 /*
- * $Id: Util.java,v 1.9 2002/01/17 19:29:15 dfs Exp $
+ * $Id: Util.java,v 1.11 2002/11/20 21:31:03 dfs Exp $
  */
 import java.util.*;
 
@@ -151,7 +151,7 @@ public final class Util {
    *         regular expression will be used to split the input.
    * @param matcher The regular expression matcher to execute the split.
    * @param pattern The regular expression to use as a split delimiter.
-   * @param input  The <code>String</code> to split.
+   * @param input   The <code>String</code> to split.
    * @param limit  The limit on the number of resulting split elements.
    *               Values <= 0 produce the same behavior as using the
    *               <b>SPLIT_ALL</b> constant which causes the limit to be 
@@ -214,7 +214,7 @@ public final class Util {
    *         occurences are appended.
    * @param matcher The regular expression matcher to execute the split.
    * @param pattern The regular expression to use as a split delimiter.
-   * @param input  The <code>String</code> to split.
+   * @param input   The <code>String</code> to split.
    * @since 2.0
    */
   public static void split(Collection results,  PatternMatcher matcher,
@@ -309,7 +309,7 @@ public final class Util {
    * {@link #split(Collection, PatternMatcher, Pattern, String)} instead.
    * @param matcher The regular expression matcher to execute the split.
    * @param pattern The regular expression to use as a split delimiter.
-   * @param input  The <code>String</code> to split.
+   * @param input   The <code>String</code> to split.
    * @return A <code>Vector</code> containing all the substrings of the input
    *         that occur between the regular expression delimiter occurences.
    * @since 1.0
@@ -333,11 +333,11 @@ public final class Util {
    * @param pattern The regular expression to search for and substitute
    *                occurrences of.
    * @param sub     The Substitution used to substitute pattern occurences.
-   * @param input  The <code>String</code> on which to perform substitutions.
-   * @param numSubs  The number of substitutions to perform.  Only the
-   *                 first <b> numSubs </b> patterns encountered are
-   *                 substituted.  If you want to substitute all occurences
-   *                 set this parameter to <b> SUBSTITUTE_ALL </b>.
+   * @param input   The <code>String</code> on which to perform substitutions.
+   * @param numSubs The number of substitutions to perform.  Only the
+   *                first <b> numSubs </b> patterns encountered are
+   *                substituted.  If you want to substitute all occurences
+   *                set this parameter to <b> SUBSTITUTE_ALL </b>.
    * @return A String comprising the input string with the substitutions,
    *         if any, made.  If no substitutions are made, the returned String
    *         is the original input String.
@@ -346,7 +346,6 @@ public final class Util {
   public static String substitute(PatternMatcher matcher, Pattern pattern,
 				  Substitution sub, String input, int numSubs)
   {
-    int substitutions;
     StringBuffer buffer = new StringBuffer(input.length());
     PatternMatcherInput pinput = new PatternMatcherInput(input);
     
@@ -372,7 +371,7 @@ public final class Util {
    * @param pattern The regular expression to search for and substitute
    *                occurrences of.
    * @param sub     The Substitution used to substitute pattern occurences.
-   * @param input  The <code>String</code> on which to perform substitutions.
+   * @param input   The <code>String</code> on which to perform substitutions.
    * @return A String comprising the input string with the substitutions,
    *         if any, made.  If no substitutions are made, the returned String
    *         is the original input String.
@@ -399,11 +398,11 @@ public final class Util {
    * @param pattern The regular expression to search for and substitute
    *                occurrences of.
    * @param sub     The Substitution used to substitute pattern occurences.
-   * @param input  The input on which to perform substitutions.
-   * @param numSubs  The number of substitutions to perform.  Only the
-   *                 first <b> numSubs </b> patterns encountered are
-   *                 substituted.  If you want to substitute all occurences
-   *                 set this parameter to <b> SUBSTITUTE_ALL </b>.
+   * @param input   The input on which to perform substitutions.
+   * @param numSubs The number of substitutions to perform.  Only the
+   *                first <b> numSubs </b> patterns encountered are
+   *                substituted.  If you want to substitute all occurences
+   *                set this parameter to <b> SUBSTITUTE_ALL </b>.
    * @return The number of substitutions made.
    * @since 2.0.5
    */
@@ -431,11 +430,11 @@ public final class Util {
    * @param pattern The regular expression to search for and substitute
    *                occurrences of.
    * @param sub     The Substitution used to substitute pattern occurences.
-   * @param input  The input on which to perform substitutions.
-   * @param numSubs  The number of substitutions to perform.  Only the
-   *                 first <b> numSubs </b> patterns encountered are
-   *                 substituted.  If you want to substitute all occurences
-   *                 set this parameter to <b> SUBSTITUTE_ALL </b>.
+   * @param input   The input on which to perform substitutions.
+   * @param numSubs The number of substitutions to perform.  Only the
+   *                first <b> numSubs </b> patterns encountered are
+   *                substituted.  If you want to substitute all occurences
+   *                set this parameter to <b> SUBSTITUTE_ALL </b>.
    * @return The number of substitutions made.
    * @since 2.0.3
    */
